@@ -1,10 +1,14 @@
-use crate::{response::IntoResponse, BoxError, Error};
+use crate::{
+    body,
+    response::{IntoResponse, Response},
+    BoxError, Error,
+};
 use bytes::Bytes;
 use futures_util::{
     ready,
     stream::{self, TryStream},
 };
-use http::{HeaderMap, Response};
+use http::HeaderMap;
 use http_body::Body;
 use pin_project_lite::pin_project;
 use std::{
@@ -77,11 +81,8 @@ where
     S::Ok: Into<Bytes>,
     S::Error: Into<BoxError>,
 {
-    type Body = Self;
-    type BodyError = Error;
-
-    fn into_response(self) -> Response<Self> {
-        Response::new(self)
+    fn into_response(self) -> Response {
+        Response::new(body::boxed(self))
     }
 }
 
