@@ -1,5 +1,6 @@
 use bytes::Bytes;
 use pin_project_lite::pin_project;
+use std::fmt;
 use std::ops::Deref;
 
 /// A string like type backed by `Bytes` making it cheap to clone.
@@ -30,6 +31,12 @@ impl ByteStr {
     }
 }
 
+impl fmt::Display for ByteStr {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.as_str())
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub(crate) struct PercentDecodedByteStr(ByteStr);
 
@@ -65,3 +72,22 @@ pin_project! {
         B { #[pin] inner: B },
     }
 }
+
+pub(crate) fn try_downcast<T, K>(k: K) -> Result<T, K>
+where
+    T: 'static,
+    K: Send + 'static,
+{
+    let mut k = Some(k);
+    if let Some(k) = <dyn std::any::Any>::downcast_mut::<Option<T>>(&mut k) {
+        Ok(k.take().unwrap())
+    } else {
+        Err(k.unwrap())
+    }
+}
+
+#[test]
+fn test_try_downcast() {
+    assert_eq!(try_downcast::<i32, _>(5_u32), Err(5_u32));
+    assert_eq!(try_downcast::<i32, _>(5_i32), Ok(5_i32));
+}
